package pnews.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jsoup.Jsoup;
import org.xml.sax.InputSource;

import com.rometools.rome.feed.synd.SyndEnclosure;
import com.rometools.rome.feed.synd.SyndEntry;
import com.rometools.rome.feed.synd.SyndFeed;
import com.rometools.rome.io.FeedException;
import com.rometools.rome.io.SyndFeedInput;

import pnews.Article;
import pnews.Category;

public class ArticleProvider {
        public final static ArticleProvider singleton = new ArticleProvider();
        private static final Logger LOG = Logger.getLogger(ArticleProvider.class.getName());
        private final Map<Category, List<Article>> articlesByCategory = new HashMap<>();
        private final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(2);
        
        private ArticleProvider() {      
                for (Category cat:Category.values())
                        scheduler.scheduleAtFixedRate(new Refresher(cat), 2, 600, TimeUnit.SECONDS);
        }
        
        private static SyndFeed getSyndFeed(String u) throws IllegalArgumentException, FeedException, MalformedURLException, IOException {
                InputStream is = new URL(u).openConnection().getInputStream();
                InputSource source = new InputSource(is);
                                
                return new SyndFeedInput().build(source);
                
        }
        
        private static Map<Category, String[]> getFeeds() {
                Map<Category, String[]> result;
                
                result = new HashMap<>();
                
                result.put(Category.TOP,
                           new String[] {
                                           "http://www.francetvinfo.fr/titres.rss",
                                           "http://www.france24.com/fr/actualites/rss",                                           
                                           "http://www.rfi.fr/general/rss",
                                           "http://www.cnews.fr/rss/une",
                                           "http://www.ladepeche.fr/rss/a-la-une.rss",
                                           "https://www.franceinter.fr/rss/a-la-une.xml",
                                           "https://www.francebleu.fr/rss/a-la-une.xml",
                                           "http://www.bfmtv.com/rss/info/flux-rss/flux-toutes-les-actualites/"
                           });
                
                result.put(Category.SPORT,
                                new String[] { "http://www.france24.com/fr/sports/rss" });
                
                result.put(Category.FRANCE,
                                new String[] { "http://www.france24.com/fr/france/rss",
                                               "http://www.rfi.fr/france/rss"});
                
                result.put(Category.EUROPE,
                                new String[] { "http://www.france24.com/fr/europe/rss" });
                
                result.put(Category.ECO,
                                new String[] { "http://www.france24.com/fr/economie/rss",
                                               "http://www.rfi.fr/economie/rss" });
                
                result.put(Category.ESSONNE,
                                new String[] { "http://www.tourisme-essonne.com/rss/actus/",
                                               "http://www.ville-palaiseau.fr/rss/actualites.htm"
                                                /*"https://www.essonneinfo.fr/feed/"*/ });
                
                result.put(Category.PEOPLE,
                                new String[] { "http://www.premiere.fr/rss/actu-live",
                                               "http://www.purepeople.com/rss/news_t0.xml"                                               
                });
                
                result.put(Category.TECHNOLOGIE,
                                new String[] { "http://feeds.feedburner.com/lesnumeriques/news",
                                               "http://www.zdnet.fr/feeds/rss/actualites/"});
                
                return result;
        }
        
        private void addArticles(Category cat, SyndFeed feed) {
                String thumbnail;
                String desc, link, title, feedTitle, feedImage;
                Date date;
                List<Article> articles;
                boolean exist;
                
                feedTitle = feed.getTitle().trim();
                
                if (feed.getImage() != null)
                        feedImage = feed.getImage().getUrl();
                else
                        feedImage = null;
                
                LOG.info("addArticles " + cat.getId() + " " + feedTitle + " number of articles: " + feed.getEntries().size());
                
                for (SyndEntry entry: feed.getEntries()) {
                        thumbnail = null;
                        for (SyndEnclosure e: entry.getEnclosures()) {
                                if (e.getType().startsWith("image/"))
                                        thumbnail = e.getUrl();    
                                break;
                        }
                        if (thumbnail == null)
                                thumbnail = feedImage;
                                
                        title = entry.getTitle().trim();
                        
                        if (entry.getDescription() != null) {                                      
                                desc = Jsoup.parse(entry.getDescription().getValue()).text();
                        } else {       
                                desc = null;
                                LOG.severe("No description for " + feedTitle + " - " + title);
                        }
                        
                        date = entry.getPublishedDate();
                        if (date == null)
                                date = entry.getUpdatedDate();
                        if (date == null) {
                                LOG.severe("The article " + feedTitle + " - " + title + " does not have a date");
                                continue;
                        }                                
                        
                        synchronized(articlesByCategory) {
                                link = entry.getLink().trim();

                                articles = articlesByCategory.get(cat);
                                exist = false;
                                if (articles == null) {
                                        articles = new ArrayList<>();
                                        articlesByCategory.put(cat, articles);
                                } else {                                
                                        for (Article a: articles)
                                                if (a.link.equals(link)) {
                                                        LOG.fine("addArticles " + link + " is already present");
                                                        exist = true;
                                                }
                                }
                                
                                if (!exist) {
                                        LOG.fine("add " + cat.getId() + " " + feedTitle + " " + title);
                                
                                        articles.add(new Article(link, cat, title, desc, thumbnail, date,
                                                        feed.getTitle()));

                                        Collections.sort(articles, new Comparator<Article>() {
                                                @Override
                                                public int compare(Article o1, Article o2) {
                                                        return o2.publicationDate.compareTo(o1.publicationDate);
                                                }

                                        });
                                } else {
                                        LOG.finest("addArticles already exist: " + title);
                                }
                        }
                }          
                
                LOG.info("addArticles done " + cat.getId());
        }
        
        private void retrieveArticles(Category cat) throws IllegalArgumentException, MalformedURLException, FeedException, IOException {
                String[] feeds;
                
                feeds = getFeeds().get(cat);
                
                if (feeds != null)
                        for (String str: feeds)
                                try {
                                        addArticles(cat, getSyndFeed(str));
                                } catch (Throwable e) {
                                        LOG.log(Level.SEVERE,
                                                "retrieveArticles failure " + cat.getId() + " " + str,
                                                e);
                                }
                else
                        LOG.severe("No feed for category " + cat);
        }
        
        public List<Article> getArticles(Category cat)
                        throws IllegalArgumentException, MalformedURLException, FeedException, IOException {
                synchronized (articlesByCategory) {
                        return articlesByCategory.get(cat);
                }
        }
        
        private class Refresher implements Runnable {
                private final Category category;
                
                public Refresher(Category category) {
                        this.category = category;
                }
                
                @Override
                public void run() {
                        List<Article> articles;
                        
                        LOG.info("refresher "+ category.getId());
                        
                        try {
                                retrieveArticles(category);
                                
                                synchronized (articlesByCategory) {
                                        articles = articlesByCategory.get(category);
                                        if (articles != null && articles.size() > 100) {
                                                articlesByCategory.put(category,
                                                                       articles.subList(0, 100));
                                                                
                                        }
                                        LOG.info("refresher " + category.getId() + " number of articles: " + articles.size());
                                }
                        } catch (IllegalArgumentException | FeedException | IOException e) {
                                LOG.log(Level.SEVERE, "refresher failure", e);
                        }                        
                        
                        LOG.info("refresher "+ category.getId() + " done");
                }                
        }
}
